/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser;

import java.io.File;
import java.io.IOException;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.json.JSONArray;
import org.json.JSONObject;

/**
 * @author Leslie Wong
 */
public class BuildDataJSONObject extends JSONObject {

	public BuildDataJSONObject() {
	}

	public BuildDataJSONObject(String json) {
		super(json);
	}

	public void addPropertiesToBuildData(String key, Properties properties) {
		JSONArray jsonArray = _toJSONArray(properties);

		put(key, jsonArray);
	}

	public void addPropertiesToBuildData(
		String key, String propertiesFilePath) {

		addPropertiesToBuildData(
			key,
			JenkinsResultsParserUtil.getProperties(
				new File(propertiesFilePath)));
	}

	public Map<String, String> getBuildDataMap(String key) {
		return getBuildDataMap(key, null);
	}

	public Map<String, String> getBuildDataMap(String key, Pattern pattern) {
		if (!has(key)) {
			throw new RuntimeException(
				"Unable to find build data for '" + key + "'");
		}

		Map<String, String> buildDataMap = new HashMap<>();

		JSONArray jsonArray = getJSONArray(key);

		for (int i = 0; i < jsonArray.length(); i++) {
			JSONObject jsonObject = jsonArray.getJSONObject(i);

			String propertyName = jsonObject.getString("name");

			if (pattern == null) {
				buildDataMap.put(propertyName, jsonObject.getString("value"));

				continue;
			}

			Matcher matcher = pattern.matcher(propertyName);

			if (matcher.matches()) {
				buildDataMap.put(propertyName, jsonObject.getString("value"));
			}
		}

		return buildDataMap;
	}

	public void writeFilteredPropertiesToFile(
			String destFilePath, Pattern pattern, String key)
		throws IOException {

		Map<String, String> buildDataMap = getBuildDataMap(key, pattern);

		StringBuilder sb = new StringBuilder();

		for (Map.Entry<String, String> entry : buildDataMap.entrySet()) {
			sb.append(entry.getKey());
			sb.append("=");
			sb.append(entry.getValue());
		}

		JenkinsResultsParserUtil.write(destFilePath, sb.toString());
	}

	public void writePropertiesToFile(String destFilePath, String key)
		throws IOException {

		writeFilteredPropertiesToFile(destFilePath, null, key);
	}

	private JSONArray _toJSONArray(Properties properties) {
		JSONArray jsonArray = new JSONArray();

		int i = 0;

		for (String propertyName : properties.stringPropertyNames()) {
			JSONObject jsonObject = new JSONObject();

			jsonObject.put("name", propertyName);
			jsonObject.put("value", properties.getProperty(propertyName));

			jsonArray.put(i, jsonObject);

			i++;
		}

		return jsonArray;
	}

}