/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser;

import java.io.File;
import java.io.IOException;

import java.util.List;

/**
 * @author Michael Hashimoto
 */
public class NPMTestBatchTestClassGroup extends BatchTestClassGroup {

	@Override
	public AxisTestClassGroup getAxisTestClassGroup(int axisId) {
		if (axisId != 0) {
			throw new IllegalArgumentException("axisId is not 0");
		}

		AxisTestClassGroup axisTestClassGroup = axisTestClassGroups.get(axisId);

		if (axisTestClassGroup != null) {
			return axisTestClassGroups.get(axisId);
		}

		return new AxisTestClassGroup(this, axisId);
	}

	protected NPMTestBatchTestClassGroup(
		String batchName, PortalGitWorkingDirectory portalGitWorkingDirectory,
		String testSuiteName) {

		super(batchName, portalGitWorkingDirectory, testSuiteName);

		_setTestClassFiles();

		_setAxisTestClassGroups();
	}

	private void _setAxisTestClassGroups() {
		if (!testClassFiles.isEmpty()) {
			AxisTestClassGroup axisTestClassGroup = new AxisTestClassGroup(
				this, 0);

			for (File testClassFile : testClassFiles) {
				axisTestClassGroup.addTestClassFile(testClassFile);
			}

			axisTestClassGroups.put(0, axisTestClassGroup);
		}
	}

	private void _setTestClassFiles() {
		if (testRelevantChanges) {
			try {
				List<File> moduleDirs =
					portalGitWorkingDirectory.
						getModifiedNPMTestModuleDirsList();

				for (File moduleDir : moduleDirs) {
					testClassFiles.add(moduleDir);
				}
			}
			catch (IOException ioe) {
				throw new RuntimeException(ioe);
			}
		}
		else {
			testClassFiles.add(
				new File(
					portalGitWorkingDirectory.getWorkingDirectory(),
					"modules"));
		}
	}

}