/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser;

/**
 * @author Michael Hashimoto
 */
public class TestClassGroupFactory {

	public static BatchTestClassGroup newBatchTestClassGroup(
		String batchName, PortalGitWorkingDirectory portalGitWorkingDirectory) {

		return newBatchTestClassGroup(
			batchName, portalGitWorkingDirectory, null);
	}

	public static BatchTestClassGroup newBatchTestClassGroup(
		String batchName, PortalGitWorkingDirectory portalGitWorkingDirectory,
		String testSuiteName) {

		if (batchName.startsWith("functional-")) {
			return new FunctionalBatchTestClassGroup(
				batchName, portalGitWorkingDirectory, testSuiteName);
		}

		if (batchName.startsWith("integration-") ||
			batchName.startsWith("unit-")) {

			return new JUnitBatchTestClassGroup(
				batchName, portalGitWorkingDirectory, testSuiteName);
		}

		if (batchName.startsWith("modules-integration-") ||
			batchName.startsWith("modules-unit-")) {

			return new ModulesJUnitBatchTestClassGroup(
				batchName, portalGitWorkingDirectory, testSuiteName);
		}

		if (batchName.startsWith("plugins-compile-")) {
			return new PluginsBatchTestClassGroup(
				batchName, portalGitWorkingDirectory, testSuiteName);
		}

		if (batchName.startsWith("portal-frontend-js-")) {
			return new NPMTestBatchTestClassGroup(
				batchName, portalGitWorkingDirectory, testSuiteName);
		}

		if (batchName.startsWith("tck-")) {
			return new TCKJunitBatchTestClassGroup(
				batchName, portalGitWorkingDirectory, testSuiteName);
		}

		return new DefaultBatchTestClassGroup(
			batchName, portalGitWorkingDirectory, testSuiteName);
	}

	public static BatchTestClassGroup newBatchTestClassGroup(
		String batchName, PortalRepositoryJob portalRepositoryJob) {

		PortalGitWorkingDirectory portalGitWorkingDirectory =
			portalRepositoryJob.getPortalGitWorkingDirectory();

		String testSuiteName = null;

		if (portalRepositoryJob instanceof PortalAcceptancePullRequestJob) {
			PortalAcceptancePullRequestJob portalAcceptancePullRequestJob =
				(PortalAcceptancePullRequestJob)portalRepositoryJob;

			testSuiteName = portalAcceptancePullRequestJob.getTestSuiteName();
		}

		return newBatchTestClassGroup(
			batchName, portalGitWorkingDirectory, testSuiteName);
	}

}