/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser.failure.message.generator;

import com.liferay.jenkins.results.parser.Build;
import com.liferay.jenkins.results.parser.Dom4JUtil;
import com.liferay.jenkins.results.parser.TopLevelBuild;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.dom4j.Element;

/**
 * @author Peter Yoo
 */
public abstract class BaseFailureMessageGenerator
	implements FailureMessageGenerator {

	@Override
	public abstract Element getMessageElement(Build build);

	protected Element getBaseBranchAnchorElement(TopLevelBuild topLevelBuild) {
		StringBuilder sb = new StringBuilder();

		sb.append("https://github.com/");

		String baseRepositoryName = topLevelBuild.getBaseRepositoryName();

		Map<String, String> baseRepositoryGitDetailsTempMap =
			topLevelBuild.getBaseGitRepositoryDetailsTempMap();

		sb.append(baseRepositoryGitDetailsTempMap.get("github.origin.name"));

		sb.append("/");
		sb.append(baseRepositoryName);
		sb.append("/tree/");
		sb.append(
			baseRepositoryGitDetailsTempMap.get("github.sender.branch.name"));

		String url = sb.toString();

		sb = new StringBuilder();

		sb.append(baseRepositoryGitDetailsTempMap.get("github.origin.name"));
		sb.append("/");
		sb.append(
			baseRepositoryGitDetailsTempMap.get("github.sender.branch.name"));

		return Dom4JUtil.getNewAnchorElement(url, sb.toString());
	}

	protected String getConsoleTextSnippet(
		String consoleText, boolean truncateTop, int end) {

		if (end == -1) {
			end = consoleText.length();
		}

		int start = getSnippetStart(consoleText, end);

		return getConsoleTextSnippet(consoleText, truncateTop, start, end);
	}

	protected String getConsoleTextSnippet(
		String consoleText, boolean truncateTop, int start, int end) {

		return "<pre><code>" +
			_getConsoleTextSnippet(consoleText, truncateTop, start, end) +
				"</code></pre>";
	}

	protected Element getConsoleTextSnippetElement(
		String consoleText, boolean truncateTop, int end) {

		if (end == -1) {
			end = consoleText.length();
		}

		int start = getSnippetStart(consoleText, end);

		return getConsoleTextSnippetElement(
			consoleText, truncateTop, start, end);
	}

	protected Element getConsoleTextSnippetElement(
		String consoleText, boolean truncateTop, int start, int end) {

		return Dom4JUtil.toCodeSnippetElement(
			_getConsoleTextSnippet(consoleText, truncateTop, start, end));
	}

	protected Element getGitCommitPluginsAnchorElement(
		TopLevelBuild topLevelBuild) {

		String repositoryName = topLevelBuild.getBaseRepositoryName();

		Map<String, String> portalRepositoryGitDetailsTempMap =
			topLevelBuild.getBaseGitRepositoryDetailsTempMap();

		Element gitCommitPluginsAnchorElement = Dom4JUtil.getNewElement("a");

		StringBuilder sb = new StringBuilder();

		sb.append("https://github.com/");
		sb.append(portalRepositoryGitDetailsTempMap.get("github.origin.name"));
		sb.append("/");
		sb.append(repositoryName);
		sb.append("/blob/");
		sb.append(
			portalRepositoryGitDetailsTempMap.get("github.sender.branch.name"));
		sb.append("/git-commit-plugins");

		gitCommitPluginsAnchorElement.addAttribute("href", sb.toString());

		gitCommitPluginsAnchorElement.addText("git-commit-plugins");

		return gitCommitPluginsAnchorElement;
	}

	protected int getSnippetStart(String consoleText, int end) {
		int start = 0;

		Matcher matcher = _pattern.matcher(consoleText);

		while (matcher.find()) {
			int x = matcher.start() + 1;

			if (x >= end) {
				return start;
			}

			start = x;
		}

		return start;
	}

	private String _getConsoleTextSnippet(
		String consoleText, boolean truncateTop, int start, int end) {

		if ((end - start) > 2500) {
			if (truncateTop) {
				start = end - 2500;

				start = consoleText.indexOf("\n", start);
			}
			else {
				end = start + 2500;

				end = consoleText.lastIndexOf("\n", end);
			}
		}

		consoleText = consoleText.substring(start, end);

		consoleText = consoleText.replaceFirst("^\\s*\\n", "");
		consoleText = consoleText.replaceFirst("\\n\\s*$", "");

		return consoleText;
	}

	private static final Pattern _pattern = Pattern.compile(
		"\\n[a-z\\-\\.]+\\:\\n");

}